import asyncio
import logging
from . import Application, TestDB
from aionsq.util import parse_addr


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()

    logging.basicConfig(level=logging.DEBUG)

    parser.add_argument('listen', default=':4181', nargs='?')
    parser.add_argument('--db', default='test', choices={'test'})
    args = parser.parse_args()
    args.listen = parse_addr(args.listen, host='0.0.0.0', proto='http')
    print(args)
    if args.db == 'test':
        db = TestDB()
    else:
        raise Exception('db not loaded')
    app = Application(db=db)
    loop = asyncio.get_event_loop()
    handler = app.make_handler()
    f = loop.create_server(handler, *args.listen)
    srv = loop.run_until_complete(f)
    print('serving on', srv.sockets[0].getsockname())
    try:
        loop.run_forever()
    except KeyboardInterrupt:
        pass
    finally:
        loop.run_until_complete(handler.finish_connections(1.0))
        srv.close()
        loop.run_until_complete(srv.wait_closed())
        loop.run_until_complete(app.finish())
    loop.close()
