import json
import logging
from aiohttp import web


class Application(web.Application):

    def __init__(self, *args, db, **kwargs):
        super().__init__(*args, **kwargs)
        self.router.add_route('GET', '/ping', self.ping_controller)
        self.router.add_route('GET', '/auth', self.auth_controller)
        self.log = logging.getLogger(__name__)
        self.db = db

    async def ping_controller(self, request):
        return web.Response(body=b'OK')

    async def auth_controller(self, request):
        self.log.info('incoming %s %s', request, request.GET)
        try:
            remote_ip = request.GET['remote_ip']
            secret = request.GET['secret']
            tls = request.GET['tls'] == 'true'
        except KeyError:
            raise web.HTTPBadRequest()

        try:
            response = await self.db.match(remote_ip, secret, tls)
        except Exception as error:
            raise web.HTTPForbidden() from error
        body = json.dumps(response).encode()
        headers = {
            'Content-Type': 'application/json',
        }
        self.log.info('resp %s %s', headers, body)
        return web.Response(body=body, headers=headers)
