from aionsq.http.client import HTTPHandler
from aionsq.http.requests import unwrapped_json, ok

__all__ = ['NSQLookup']


class NSQLookup(HTTPHandler):

    async def lookup(self, *, topic):
        """Returns a list of producers for a topic.

        Parameters:
            topic (str): the topic to list producers for

        """
        params = {'topic': topic}
        path = '/lookup'
        response = await self.req_handler('GET', path, params=params)
        result = await unwrapped_json(response)
        return result

    async def topics(self):
        """Returns a list of all known topics.
        """
        path = '/topics'
        response = await self.req_handler('GET', path)
        result = await unwrapped_json(response)
        return result

    async def channels(self, *, topic):
        """Returns a list of all known channels of a topic.

        Parameters:
            topic (str): the topic to list channels for
        """
        params = {'topic': topic}
        path = '/channels'
        response = await self.req_handler('GET', path, params=params)
        result = await unwrapped_json(response)
        return result

    async def nodes(self):
        """Returns a list of all known nsqd.
        """
        path = '/nodes'
        response = await self.req_handler('GET', path)
        result = await unwrapped_json(response)
        return result

    async def delete_topic(self, *, topic):
        """Deletes an existing topic.

        Parameters:
            topic (str): the existing topic to delete
        """
        params = {'topic': topic}
        path = '/delete_topic'
        response = await self.req_handler('GET', path, params=params)
        result = await response.json()
        return result

    async def delete_channel(self, *, topic, channel):
        """Deletes an existing channel of an existing topic.

        Parameters:
            topic (str): the existing topic
            channel (str): the existing channel to delete
        """
        params = {'topic': topic,
                  'channel': channel}
        path = '/delete_channel'
        response = await self.req_handler('GET', path, params=params)
        result = await response.json()
        return result

    async def tombstone_topic_producer(self, *, topic, node):
        """Tombstones a specific producer of an existing topic.

        Parameters:
            topic (str): the existing topic
            node (str): the producer (nsqd) to tombstone
                        (identified by <broadcast_address>:<http_port>)
        """
        params = {'topic': topic,
                  'node': node}
        path = '/tombstone_topic_producer'
        response = await self.req_handler('GET', path, params=params)
        result = await response.json()
        return result

    async def ping(self):
        """Monitoring endpoint, should return OK.
        """
        path = '/ping'
        response = await self.req_handler('GET', path)
        result = await ok(response)
        return result

    async def info(self):
        """Returns version information.
        """
        path = '/info'
        response = await self.req_handler('GET', path)
        result = await unwrapped_json(response)
        return result
