import logging
try:
    import snappy
except ImportError:
    pass
import zlib


class DeflateHandler:

    def __init__(self, level):
        wbits = -zlib.MAX_WBITS
        self._compressor = zlib.compressobj(level, zlib.DEFLATED, wbits)
        self._decompressor = zlib.decompressobj(wbits)
        self.log = logging.getLogger(__name__)

    def compress(self, data):
        chunk = self._compressor.compress(data)
        return chunk + self._compressor.flush(zlib.Z_SYNC_FLUSH)

    def decompress(self, chunk):
        try:
            data = self._decompressor.decompress(chunk)
            self.log.debug('decompress %s to %s', chunk, data)
            return data
        except zlib.error:
            self.log.warning('unable to decompress %s' % chunk)
            raise


class SnappyHandler:

    def __init__(self):
        try:
            self._compressor = snappy.StreamCompressor()
            self._decompressor = snappy.StreamDecompressor()
        except Exception as error:
            raise RuntimeError('snappy required') from error
        self.log = logging.getLogger(__name__)

    def compress(self, data):
        return self._compressor.add_chunk(data, compress=True)

    def decompress(self, chunk):
        try:
            data = self._decompressor.decompress(chunk)
            self.log.debug('decompress %s to %s', chunk, data)
            return data
        except Exception:
            self.log.warning('unable to decompress %s' % chunk)
            raise
